<?php

namespace Drupal\responsive_voice_tts\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\node\NodeInterface;
use Drupal\responsive_voice_tts\SpeakerService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'ResponsiveVoiceBlock' block.
 *
 * @Block(
 *   id = "responsive_voice_block",
 *   admin_label = @Translation("Responsive Voice Block"),
 * )
 */
class ResponsiveVoiceTtsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The speaker service for TTS functionality.
   *
   * @var \Drupal\responsive_voice_tts\SpeakerService
   */
  protected $speakerService;

  /**
   * The configuration factory service.
   *
   * This is used to retrieve the module's configuration settings.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, AccountProxyInterface $currentUser, EntityTypeManagerInterface $entityTypeManager, SpeakerService $speaker_service, ConfigFactoryInterface $config_factory, CurrentRouteMatch $currentRouteMatch) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentUser = $currentUser;
    $this->entityTypeManager = $entityTypeManager;
    $this->speakerService = $speaker_service;
    $this->configFactory = $config_factory;
    $this->currentRouteMatch = $currentRouteMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('responsive_voice_tts.speaker_service'),
      $container->get('config.factory'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $node = $this->currentRouteMatch->getParameter('node');
    if ($node instanceof NodeInterface) {
      $config = $this->configFactory->get('responsive_voice_tts.settings');
      $enabled_content_types = $config->get('content_types') ?? [];
      if (in_array($node->getType(), $enabled_content_types)) {
        $body = $node->get('body')->value;

        $output = $this->speakerService->generateSpeechOutput(addslashes(strip_tags($body)));
        $build['content'] = $output;
        $build['#cache'] = [
          'contexts' => ['user', 'session', 'route', 'url'],
          'max-age' => 0,
          'tags' => ['node:' . $node->id()],
        ];
        return $build;
      }
    }
    return [];
  }

}
